# main.py
#
# Copyright 2025 Kate
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
import gi

gi.require_version('Gtk', '4.0')
gi.require_version('Adw', '1')

from gi.repository import Gtk, Gio, Adw
from .window import IdentitiesWindow


class IdentitiesApplication(Adw.Application):
    """The main application singleton class."""

    def __init__(self):
        super().__init__(application_id='one.k8ie.Identities',
                         flags=Gio.ApplicationFlags.DEFAULT_FLAGS)
        self.create_action('quit', lambda *_: self.quit(), ['<primary>q'])
        self.create_action('about', self.on_about_action)
        self.create_action('preferences', self.on_preferences_action)

    def do_activate(self):
        """Called when the application is activated.

        We raise the application's main window, creating it if
        necessary.
        """
        win = self.props.active_window
        if not win:
            win = IdentitiesWindow(application=self)
        win.present()

    def on_about_action(self, widget, _):
        """Callback for the app.about action."""
        about = Adw.AboutDialog(application_name='Identities',
                                application_icon='one.k8ie.Identities',
                                developer_name='Kateřina Medvědová',
                                version='0.1.3',
                                developers=['Kateřina Medvědová https://github.com/k8ieone'],
                                copyright='© 2025 Kateřina Medvědová',
                                license_type=Gtk.License(3),
                                website='https://github.com/k8ieone/identities',
                                issue_url='https://github.com/k8ieone/identities/issues')
        about.add_credit_section("Inspiration", [
                                 "Caerbannog https://git.sr.ht/~craftyguy/caerbannog",
                                 "passpy https://github.com/bfrascher/passpy"
                                 ])
        about.add_credit_section("Used libraries", [
                                 "pyotp https://github.com/pyauth/pyotp",
                                 ])
        about.present(parent=self.props.active_window)

    def on_preferences_action(self, widget, _):
        """Callback for the app.preferences action."""
        print('app.preferences action activated')

    def create_action(self, name, callback, shortcuts=None):
        """Add an application action.

        Args:
            name: the name of the action
            callback: the function to be called when the action is
              activated
            shortcuts: an optional list of accelerators
        """
        action = Gio.SimpleAction.new(name, None)
        action.connect("activate", callback)
        self.add_action(action)
        if shortcuts:
            self.set_accels_for_action(f"app.{name}", shortcuts)


def main(version):
    """The application's entry point."""
    app = IdentitiesApplication()
    return app.run(sys.argv)
